"""Setup.py entry point for package.

Configuration is handled by setuptools>30.3.0 through setup.cfg.
https://setuptools.readthedocs.io/en/latest/setuptools.html#metadata
https://setuptools.readthedocs.io/en/latest/setuptools.html#options
"""

import setuptools


setuptools.setup(
    package_dir={'': 'src'},
)
